function [ Solution, Score, gbest_history ] = PSO( particles, ...
                                      dimension,...
                                      Limits, ...
                                      generations, ...
                                      Eval,...
                                      convergence,...
                                      acc_coeffs);
                                  
    %%% PSO is configured to minimise the score              
    tic
    if ((nargin < 6) || (isempty(convergence)))
        convergence = false;
    end
    if ((nargin < 7) || (isempty(acc_coeffs)))
        acc_coeffs = [0.729, 1.494];
    end
    % Maximum allowable propagation delay
    
    global num_errors 
    num_errors = 0;
    
    % Parameter setting
    INERTIA = acc_coeffs(1);
    COGNITION = acc_coeffs(2);
    SOCIAL = acc_coeffs(2);
    
    % Variable initialisation
    d = dimension;
    n = particles;
    g = generations;
    minimum = Limits(1);
    maximum = Limits(2);
    maxVel = Limits(3);

    % Initial Values
    rand_vel = @(n) random_velocity(minimum, maximum, maxVel, d,n);
    rand_pos = @(len) minimum + (maximum-minimum) * rand(d, len);
    
    vel=rand_vel(n);
    swarm = rand_pos(n); % Each col vector represents a particle solution
    
    pbest = Inf((d + 3), n); % The personal best solution of each particle
    
    gbest = Inf((d + 3), 1); % The globally best solution
    gbest_history = zeros((d + 3), g); %Store the gbest for each generation
    
    done = false;
    gen_num = 1;
    while ~done % Looping through each generation
        % Update personal bests
        
        area = zeros(1,n);
        time = zeros(1,n);
        score = zeros(1,n);
        for particle = 1:n
           [score(particle), area(particle), time(particle)] = Eval(swarm(:,particle));
        end
        
        unfit = logical(score == Inf);
        
        improved_particles = score<=pbest(end,:); %We are typing to minimise the score (like golf)
        swarm_results = [swarm; area; time; score];
        pbest(:,improved_particles) = swarm_results(:,improved_particles);
        
        no_pbest = pbest(d+1,:)==Inf;
        

        % Update global best
        [~,best_pbest] = min(pbest(end,:)); %Find which column, the best pbest fitenss was max
        gbest = pbest(:,best_pbest);
        gbest_history(:,gen_num) = gbest;
        
       
        p_displace = (pbest(1:d,:)-swarm);
        g_displace = (bsxfun(@minus,gbest(1:d,1),swarm));
        
        i_vel = INERTIA*vel;
        %i_vel(:,unfit & no_pbest)=rand_vel(nnz(unfit& no_pbest)); %If moved then the interia we built up is not useful anymore
        
        p_vel =  COGNITION * rand_factor(d,n) .* p_displace;
        %p_vel(:, unfit & no_pbest)=0; %If we full reset it then don't use COGNITITIVE velocity as it has no meaningly pbest
        
        g_vel = SOCIAL * rand_factor(d,n) .* g_displace;
        
        vel = i_vel + p_vel + g_vel;  
        vel = min(max(vel,-maxVel),maxVel); %CLIP values
        
        vel(:,unfit & no_pbest)=rand_vel(nnz(unfit& no_pbest));
        
        % Update Position
        swarm = max( min( (swarm+vel), maximum),minimum);
		% If a particle is not fit and never has found a pbest, then it teleports randomly
        swarm(:,unfit & no_pbest)=rand_pos(nnz(unfit& no_pbest));
        
        disp(gen_num)
        gen_num = gen_num + 1;
        
        if (rem(gen_num,10)==0)
            disp(gbest(:,end))
        end
        
        if (gen_num > g)
            done = true;
        end
        
        if (convergence)
            deviation = getSwarmDeviation(swarm)
            if (deviation < 1)
                done = true;
            end
        end
        
    end
    Solution = gbest(1:d);
    Score = gbest(end);
    
	% Using the Silicon area based evaluation, the score is much smaller than the area if the delay is appropriate
	%siliconArea = sum(Solution * minimum);
    %if (siliconArea < -Score)
		%prop = ((Score + 21600)/siliconArea)*2.5*10^-9;
        %disp('Failed to optimise');
        %fprintf('Got a propagation delay of %e\n',prop);
    %end
    toc
end

function [random_vals] = rand_factor(d,n)
    %%Returns a factor matrix suitable for causing row-wise multipciation
    entries = rand(1, n); %Per particle a random factor same for in all dimentions (thus d does not appear here)
    random_vals = repmat(entries,d,1); % use that same factor in each dimention
end

function [vel] = random_velocity(minimumPos,maximumPos, maxVel, nDim, nParticles)
    vel = minimumPos + (maximumPos-minimumPos)*0.5*rand(nDim, nParticles); % Each col vector represents a particle's velocity
    vel = min(vel,maxVel); % Clip to be less than maxVel
    vel = (-1).^randi(2,nDim,nParticles).*vel; %Randomise direction
end
    